﻿Public Class StackOfCommand
    Inherits Stack(Of ICommand)
    Public ReadOnly Property IsEmpty As Boolean
        Get
            Return Me.Count = 0
        End Get
    End Property
End Class

Public Class UndoManager
    Public Event PropertyChange()
    Private UndoStack As StackOfCommand
    Private RedoStack As StackOfCommand

    Public Sub New()
        UndoStack = New StackOfCommand
        RedoStack = New StackOfCommand
    End Sub

    Public Sub Execute(aCommand As ICommand)
        Try
            Dim aOldState As New OldState(Me)
            aCommand.Execute()
            If IsUndoAvailable AndAlso UndoStack.Peek.IsCollapsible(aCommand) Then
                UndoStack.Peek.Collapse(aCommand)
            Else
                UndoStack.Push(aCommand)
            End If
            RedoStack.Clear()
            fireChanges(aOldState)
        Catch ex As Exception

        End Try
    End Sub
    Public Sub Undo()
        If Not UndoStack.IsEmpty Then
            Dim aOldState As New OldState(Me)
            Dim aCommand As ICommand = UndoStack.Pop
            aCommand.Undo()
            RedoStack.Push(aCommand)
            FireChanges(aOldState)
        End If
    End Sub
    Public Sub Redo()
        If Not RedoStack.IsEmpty Then
            Dim aOldState As New OldState(Me)
            Dim aCommand As ICommand = RedoStack.Pop
            aCommand.Redo()
            UndoStack.Push(aCommand)
            FireChanges(aOldState)
        End If
    End Sub
    Private Sub FireChanges(aOldState As OldState)
        RaiseEvent PropertyChange()
    End Sub

    Public ReadOnly Property IsUndoAvailable As Boolean
        Get
            Return Not UndoStack.IsEmpty
        End Get
    End Property
    Public ReadOnly Property IsRedoAvailable As Boolean
        Get
            Return Not (RedoStack.Count = 0)
        End Get
    End Property
    Public ReadOnly Property UndoName As String
        Get
            If IsUndoAvailable Then Return UndoStack.Peek.Name
            Return ""
        End Get
    End Property
    Public ReadOnly Property RedoName As String
        Get
            If IsRedoAvailable Then Return RedoStack.Peek.Name
            Return ""
        End Get
    End Property

    Private Class OldState
        Public UndoName As String
        Public RedoName As String
        Public undoAvailable As Boolean
        Public redoAvailable As Boolean
        Public Sub New(um As UndoManager)
            Me.UndoName = um.UndoName
            Me.RedoName = um.RedoName
            Me.undoAvailable = um.IsUndoAvailable
            Me.redoAvailable = um.IsRedoAvailable
        End Sub
    End Class
End Class
